from copy import copy
from unittest.mock import MagicMock


def test_perfect_accuracy(mocked_app):
    win = MagicMock()

    for character in copy(mocked_app.text):
        mocked_app.typing_mode(win, character)
    assert mocked_app.accuracy == 100


def test_error_during_typing(mocked_app):
    win = MagicMock()

    text = 2 * ["z", "KEY_BACKSPACE"] + list(copy(mocked_app.text))
    print("list is ", mocked_app.text)
    for character in copy(text):
        mocked_app.typing_mode(win, character)
    assert mocked_app.accuracy == 98.75


def test_same_character_after_typing(mocked_app):
    win = MagicMock()

    text = list(copy(mocked_app.text)) + list(mocked_app.text[-1] * 10)

    for character in copy(text):
        mocked_app.typing_mode(win, character)
    assert mocked_app.accuracy == 100



import pytest

from mitype import history


class TestShowHistory:
    def test_show_history_no_file(self, monkeypatch, tmp_path):
        history_file = tmp_path / ".mitype_history.csv"

        monkeypatch.setattr(history, "history_file_absolute_path", lambda: history_file)
        history.show_history(10)

    def test_show_history_file_empty(self, empty_history_file):
        history.show_history(10)


class TestGetHistoryRecords:
    def test_get_history_no_file(self, monkeypatch, tmp_path):
        history_file = tmp_path / ".mitype_history.csv"

        monkeypatch.setattr(history, "history_file_absolute_path", lambda: history_file)
        assert history.get_history_records(-1) == []

    def test_get_history_empty(self, empty_history_file):
        assert history.get_history_records(-1) == []

    def test_get_history_with_header_only(self, header_only_history_file):
        assert history.get_history_records(-1) == []

    def test_get_history(self, history_file):
        records = history.get_history_records(-1)

        assert len(records) == 100

    def test_get_history_only_some_records(self, history_file):
        records = history.get_history_records(10)

        assert len(records) == 10
